c*******************************************************************
c* Short description                                               *
c*   Library of elastic transport routines for electrons           *
c*                                                                 *
c* Dependencies:                                                   *
c*   -> uses libmat.f                                              *
c*   -> uses libmath.f                                             *
c*   -> uses libpenmath.f                                          *
c*                                                                 *
c* Last update                                                     *
c*   2001-02-14 JS. newset() substituted by newpaw().              *
c*   1998-04-12 JS. Created.                                       *
c*******************************************************************



c*******************************************************************
c*******************************************************************
c*   Goudsmit & Saunderson-related routines                        *
c*******************************************************************
c*******************************************************************


      subroutine scatgs(energy,path,mu,weight)
c*******************************************************************
c*    Samples cos(theta) and gives the weight factor for the       *
c*    outgoing electron according to the GS distribution           *
c*                                                                 *
c*    Input:                                                       *
c*      energy -> kinetic energy (eV) of the electron              *
c*      path -> path length in g/cm^2                              *
c*    Output:                                                      *
c*      mu -> polar angle -cos(theta)-                             *
c*      weight -> q-function value associated to the delivered mu  *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> inirng() must be called before first call               *
c*******************************************************************
      implicit none
      real*8 energy,mu,weight,path
      real*8 aw,spread,bw,onebw,u,rng,qgs

      call spregs(energy,path,aw,spread,bw)
      onebw = 1.d0+bw
      u = rng()
      mu = (onebw-u*(onebw+bw))/(onebw-u)
      weight = qgs(u)
      end


      real*8 function qsurf(u,bw)
c*******************************************************************
c*    MS q function in u space                                     *
c*                                                                 *
c*    Input:                                                       *
c*      u -> angular variable                                      *
c*      bw -> spreading parameter times aw, the screening parameter*
c*    Comments                                                     *
c*      -> newpaw() must be called before when path or aw change   *
c*      -> qsurf() works much faster if succesive calls change u   *
c*         but keep the set {path,aw}                              *
c*******************************************************************
      implicit none
      integer*4 n
      real*8 u,bw,denom,mu,fgs

      denom = 1.d0/(1.d0+bw-u)
      mu = 1.d0-2.d0*bw*u*denom
      call gs(mu,fgs,n)
      qsurf = fgs*2.d0*bw*(1.d0+bw)*denom*denom
      end


      subroutine gs(mu,fgs,n)
c*******************************************************************
c*    Calculates the Goudsmith-Saunderson distribution for a       *
c*    ScreenRuther DCS. The value returned corresponds to a certain*
c*    value of the angle and the parameters defining the process.  *
c*                                                                 *
c*    Input:                                                       *
c*      mu -> cos(theta) -theta = polar angle-                     *
c*    Output:                                                      *
c*      fgs -> probability density at mu                           *
c*      n -> order of the last term added in the series expansion  *
c*    Comments                                                     *
c*      -> newpaw() must be called first when path or aw change    *
c*      -> gs() works much faster if succesive calls change mu     *
c*         but keep the set {path,aw}                              *
c*******************************************************************
      implicit none
      integer*4 n
      real*8 mu,fgs
      integer conver
      real*8 kcoef

      conver = 0
      call inileg(mu,kcoef(0),fgs)
      n = 1
 10   continue
        call legsum(kcoef(n),fgs,conver)
        if (conver.ge.3) then
          fgs = fgs/kcoef(0)
          return
        endif
        n = n+1
      goto 10
      end


      subroutine spreaw(spr,bw)
c*******************************************************************
c*    Optimized spreading parameter bw/aw. The output value        *
c*    achieves the flatest q(u) surface.                           *
c*                                                                 *
c*    Output:                                                      *
c*      spr -> spreading parameter bw/aw                           *
c*      bw -> broad screening parameter to get the flattest q surf *
c*    Comments:                                                    *
c*      -> newpaw() must be called first when path or aw change    *
c*******************************************************************
      implicit none
      real*8 spr,bw
      real*8 path,aw,acpath,epath
      common /pthdat/ path,aw,acpath,epath
      integer cnum,cden
      integer*4 n,nplus1,nplus2
      real*8 numer,denom,kbw,kcoef
      real*8 kn,kn1,kn2,sumden,tosuma,tosumc,tosumn,tosumd
      real*8 m2thir,zero,accu
      parameter (m2thir=-0.6666666666666666d0)
      parameter (zero=1.d-30,accu=1.d-10)

      kn  = kcoef(0)
      kn1 = kcoef(1)
      kn2 = kcoef(2)
      cnum = 0
      cden = 0
      numer = 0.d0
      sumden = 0.d0
      n = 0
com: Adding up the numerator and denominator series
 10   continue
        nplus1 = n+1
        nplus2 = n+2
        tosumd = nplus1*kn*kn1
com: Denominator converged?
        if (cden.eq.0) then
          if (tosumd.lt.accu*sumden) cden = 1
          sumden = sumden+tosumd
        endif
com: Numerator converged?
        if (cnum.lt.3) then
          tosuma = nplus1*dble(nplus2)/(2.d0*n+3.d0)*kn*kn2
          tosumc=1.5d0*(n+0.5d0)*(m2thir+n*dble(nplus1))/
     &           ((n-0.5d0)*(n+1.5d0))*kn*kn
          tosumn = tosuma+tosumc-2.d0*tosumd
          if (dabs(tosumn).lt.accu*dabs(numer)) then
            cnum = cnum+1
          else
            cnum = 0
          endif
          numer = numer+tosumn
        endif
com: Have both numerator and denominator converged?
        if (cden*cnum.eq.3) then
          denom = 4.d0*sumden
          if (denom.lt.zero) then
            bw = 1.d0/zero
            spr = bw/aw
            return
          endif
          kbw = numer/denom
          bw = kbw+dsqrt(kbw*(kbw+1.d0))
          spr = bw/aw
          return
        endif
        kn = kn1
        kn1 = kn2
        kn2 = kcoef(nplus2+1)
        n = nplus1
      goto 10
      end


      subroutine newset(pathn,awn)
c*******************************************************************
c*    Obsolete.                                                    *
c*******************************************************************
      real*8 pathn,awn
      write(*,*) 'newset:error: Obsolete; use newpaw() instead.'
      stop
      end


      subroutine newpaw(pathn,awn,kil1st)
c*******************************************************************
c*    Initializes kcoef() every time either path or aw change      *
c*                                                                 *
c*    Input:                                                       *
c*      pathn -> path length in MFP units                          *
c*      awn -> screening parameter: ScreenRutherDCS~(1+2*aw-mu)^-2 *
c*      kil1st -> 1 to substract 1st collision, 0 else             *
c*******************************************************************
      implicit none
      integer kil1st
      real*8 pathn,awn
      integer kill1
      integer*4 maxnk,nk
      real*8 k
      parameter (maxnk=20000)
      common /gsdat/ k(maxnk),nk,kill1
      real*8 path,aw,acpath,epath
      common /pthdat/ path,aw,acpath,epath
      real*8 laccu
      parameter (laccu=1.d-15)

      path = pathn
      aw = awn
      epath = dexp(-path)
      acpath = laccu*epath
      if (kil1st.ne.0.and.kil1st.ne.1) then
        write(*,*) 'newpaw:error: Invalid argument: kil1st:'
        write(*,*) kil1st
        stop
      endif
      kill1 = kil1st
      nk = 0
      call iniwtc(1,aw)
      end


      real*8 function kcoef(n)
c*******************************************************************
c*    Gives the n-th coefficient of the GS series expansion, that  *
c*    is, the exp(-path/lambdan), lambdan being the inverse n-th   *
c*    trasport MFP.                                                *
c*                                                                 *
c*    Input:                                                       *
c*      n -> order of coefficient to be calculated                 *
c*    Comments:                                                    *
c*      -> newpaw() must be called first when path or aw change    *
c*      -> it stores previously calculated values in a common stack*
c*******************************************************************
      implicit none
      integer*4 n
      integer kill1
      integer*4 maxnk,nk
      real*8 k
      parameter (maxnk=20000)
      common /gsdat/ k(maxnk),nk,kill1
      real*8 path,aw,acpath,epath
      common /pthdat/ path,aw,acpath,epath
      integer*4 i
      real*8 wtc

      if (n.eq.0) then
        kcoef = 1.d0-epath*kill1
        return
      endif
      if (n.gt.maxnk) then
        write(*,*) 'kcoef Error: stack overflow -> enlarge maxnk'
        stop
      endif
      if (n.gt.nk) then
        do 10 i=nk+1,n
 10     k(i) = dexp(-path*wtc())-epath*kill1
        nk = n
      endif
      kcoef = k(n)
      end


      real*8 function wtc()
c*******************************************************************
c*    Calculates the transport coefficients associated to the      *
c*    ScreenRutherford single scattering model. Uses the aprox     *
c*    g_n = 1 - z K1(z) (1 - 2 n (n+1) aw (PolGam(n) -             *
c*         1/2 Ln(n(n+1))))                                        *
c*                                                                 *
c*     where z = 2*Sqrt(n*(n+1)*aw) and                            *
c*           PolGam(n) = Sum(1/k,k=1,n) - EulerG                   *
c*           K1 modified Bessel function                           *
c*     This expression reproduces the exact result for             *
c*     the distribution with a maximum deviation of the            *
c*     order of aw/4 (i.e. for aw < 0.01 to better than 0.3%)      *
c*                                                                 *
c*    Output:                                                      *
c*      wtc -> g_n = 1-<Pn> for ScreenRutherford DCS               *
c*    Comments:                                                    *
c*      -> iniwtc() must be called before when aw changes          *
c*      -> each call returns a new term, starting at n -see iniwtc-*
c*      -> uses small-momentum aprox                               *
c*******************************************************************
      implicit none
      integer*4 n
      real*8 aw,polgam
      common /wtcd/ aw,polgam,n
      integer*4 nplus1
      real*8 dbesk1,nn1,z

      if (n.eq.0) then
        wtc = 0.d0
        n = 1
        return
      endif
      polgam = polgam+1.d0/n
      nplus1 = n+1
      nn1 = n*dble(nplus1)
      z = 2.d0*dsqrt(nn1*aw)
      wtc = 1.d0-z*dbesk1(z)*
     &     (1.d0+aw*2.d0*nn1*(polgam-0.5d0*dlog(nn1)))
      n = nplus1
      end


      subroutine iniwtc(n0,awval)
c*******************************************************************
c*    Initializes data to be used by wtc()                         *
c*                                                                 *
c*    Input:                                                       *
c*      n0 -> identifies the first term returned by wtc()          *
c*      awval -> screening parameter:ScreenRutherDCS~(1+2*aw-mu)^-2*
c*    Comments:                                                    *
c*      -> data is transferred to wtc through a common block       *
c*******************************************************************
      integer*4 n0
      real*8 awval
      integer*4 n
      real*8 aw,polgam
      common /wtcd/ aw,polgam,n
      real*8 eulerc
      parameter (eulerc=0.57721566490153286d0)
      integer*4 i

      aw = awval
      n = n0
      polgam = 0.d0
      do 10 i=n-1,1,-1
        polgam = polgam+1.d0/i
 10   continue
      polgam = polgam-eulerc
      end


      real*8 function qgs(u)
c*******************************************************************
c*    MS q function in terms of u and the kinetic energy           *
c*                                                                 *
c*    Comments:                                                    *
c*      -> spregs() must be called before when the energy changes  *
c*      -> qgs() works much faster if succesive calls change u     *
c*         but keep the energy unchanged                           *
c*      -> you will probably prefer to code this function inline;  *
c*         it has been included for completeness only              *
c*******************************************************************
      implicit none
      real*8 u,qsurf
      real*8 bwq
      common /qec/ bwq

      qgs = qsurf(u,bwq)
      end


      subroutine spregs(energy,path,aw,spr,bw)
c*******************************************************************
c*    Optimized spreading parameter as a funtion of the kinetic    *
c*    energy.                                                      *
c*                                                                 *
c*    Input:                                                       *
c*      energy -> kinetic energy in eV                             *
c*      path -> path length in g/cm^2                              *
c*    Output:                                                      *
c*      aw -> screening parameter                                  *
c*      spr -> spreading parameter                                 *
c*      bw -> broad screening parameter=aw*spr                     *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> the 1st collision term is substracted --see call to     *
c*         newpaw() if you want to change that.                    *
c*******************************************************************
      implicit none
      real*8 energy,aw,spr,bw,path
      real*8 bwq
      common /qec/ bwq
      real*8 lambda,awf

      aw = awf(energy)
      call newpaw(path/lambda(energy),aw,1)
      call spreaw(spr,bw)
      bwq = bw
      end


c*******************************************************************
c*******************************************************************
c* Lewis-related routines                                          *
c* These routines calculate the MS pdf taking into account the     *
c* energy losses within the CSDA aprox (hence, libeloss.f is used).*
c*                                                                 *
c* Lewis routines may call some of the G&S ones, but the opposite  *
c* does not happen                                                 *
c*******************************************************************
c*******************************************************************


      subroutine scatle(energy,path,mu,weight)
c*******************************************************************
c*    Samples cos(theta) and gives the weight factor for the       *
c*    outgoing electron according to the Lewis distribution        *
c*                                                                 *
c*    Input:                                                       *
c*      energy -> kinetic energy (eV) of the electron              *
c*      path -> path length in g/cm^2                              *
c*    Output:                                                      *
c*      mu -> polar angle -cos(theta)-                             *
c*      weight -> q-function value associated to the delivered mu  *
c*    Comments:                                                    *
c*      -> initot() must be called before 1st call                 *
c*      -> inirng() must be called before 1st call                 *
c*******************************************************************
      implicit none
      real*8 energy,mu,weight,path
      real*8 aw,spread,bw,onebw,u,rng,qlewis

      call newene(energy,path)
      call sprele(energy,aw,spread,bw)
      onebw = 1.d0+bw
      u = rng()
      mu = (onebw-u*(onebw+bw))/(onebw-u)
      weight = qlewis(u)
      end


      real*8 function qlewis(u)
c*******************************************************************
c*    MS q function in terms of u and the kinetic energy,          *
c*    under the assumption of CSDA energy losses                   *
c*                                                                 *
c*    Comments:                                                    *
c*      -> sprele() must be called before when the energy or path  *
c*         length change.                                          *
c*      -> qlewis() works much faster if succesive calls change u  *
c*         but keep other variables unchanged.                     *
c*******************************************************************
      implicit none
      real*8 u
      real*8 bwlew
      common /qlew/ bwlew
      integer*4 n
      real*8 denom,lew,mu

      denom = 1.d0/(1.d0+bwlew-u)
      mu = 1.d0-2.d0*bwlew*u*denom
      call lewis(mu,lew,n)
      qlewis = lew*2.d0*bwlew*(1.d0+bwlew)*denom*denom
      end


      subroutine lewis(mu,lew,n)
c*******************************************************************
c*    Calculates the Lewis' multiple scattering PDF for a          *
c*    ScreenRuther DCS. The value returned corresponds to a certain*
c*    value of the angle and the parameters defining the process.  *
c*                                                                 *
c*    Input:                                                       *
c*      mu -> cos(theta) -theta = polar angle-                     *
c*    Output:                                                      *
c*      lew -> probability density at mu                           *
c*      n -> order of the last term added in the series expansion  *
c*    Comments                                                     *
c*      -> inimat() must be called before first call               *
c*      -> newene() must be called first each time the energy or   *
c*         path length change.                                     *
c*      -> lewis() works much faster if succesive calls change mu  *
c*         but keep the energy unchanged                           *
c*      -> lewis() is a copy of gs(), except that calls to kcoef() *
c*         have been substituted by calls to kcoefL()              *
c*******************************************************************
      implicit none
      integer*4 n
      real*8 mu,lew
      integer conver
      real*8 kcoefL

      conver = 0
      call inileg(mu,dexp(-kcoefL(0)),lew)
      n = 1
 10   continue
        call legsum(dexp(-kcoefL(n)),lew,conver)
        if (conver.ge.3) then
          lew = lew/dexp(-kcoefL(0))
          return
        endif
        n = n+1
      goto 10
      end


      subroutine sprele(energy,aw,spread,bw)
c*******************************************************************
c*    Optimized spreading parameter bw/aw. The output value        *
c*    achieves the flatest q(u) surface. This routine takes into   *
c*    account energy losses within the CSDA aprox.                 *
c*                                                                 *
c*    Input:                                                       *
c*      energy -> initial kinetic energy in eV                     *
c*    Output:                                                      *
c*      aw -> screening parameter at E=initial_energy              *
c*      bw -> broad screening parameter to get the flattest q surf *
c*      spread -> spreading parameter bw/aw                        *
c*    Comments:                                                    *
c*      -> initot() must be called before 1st call                 *
c*      -> newene() must be called first                           *
c*      -> sprele() is similar to spreaw(), except that calls to   *
c*         kcoef() are replaced by calls to the CSDA-like kcoefL() *
c*******************************************************************
      implicit none
      real*8 energy,aw,spread,bw
      real*8 bwlew
      common /qlew/ bwlew
      integer cnum,cden
      integer*4 n,nplus1,nplus2
      real*8 numer,denom,kbw,kcoefL
      real*8 kn,kn1,kn2,sumden,tosuma,tosumc,tosumn,tosumd
      real*8 m2thir,zero,accu
      parameter (m2thir=-0.6666666666666666d0)
      parameter (zero=1.d-30,accu=1.d-10)
      real*8 awtot

      aw = awtot(energy)
      kn  = dexp(-kcoefL(0))
      kn1 = dexp(-kcoefL(1))
      kn2 = dexp(-kcoefL(2))
      cnum = 0
      cden = 0
      numer = 0.d0
      sumden = 0.d0
      n = 0
com: Adding up the numerator and denominator series
 10   continue
        nplus1 = n+1
        nplus2 = n+2
        tosumd = nplus1*kn*kn1
com: Denominator converged?
        if (cden.eq.0) then
          if (tosumd.le.accu*sumden) cden = 1
          sumden = sumden+tosumd
        endif
com: Numerator converged?
        if (cnum.lt.3) then
          tosuma = nplus1*dble(nplus2)/(2.d0*n+3.d0)*kn*kn2
          tosumc=1.5d0*(n+0.5d0)*(m2thir+n*dble(nplus1))/
     &           ((n-0.5d0)*(n+1.5d0))*kn*kn
          tosumn = tosuma+tosumc-2.d0*tosumd
          if (dabs(tosumn).le.accu*dabs(numer)) then
            cnum = cnum+1
          else
            cnum = 0
          endif
          numer = numer+tosumn
        endif
com: Have both numerator and denominator converged?
        if (cden*cnum.eq.3) then
          denom = 4.d0*sumden
          if (denom.lt.zero) then
            bw = 1.d0/zero
            spread = bw
            bwlew = bw
            return
          endif
          kbw = numer/denom
          bw = kbw+dsqrt(kbw*(kbw+1.d0))
          bwlew = bw
          spread = bw/aw
          return
        endif
        kn = kn1
        kn1 = kn2
        kn2 = dexp(-kcoefL(nplus2+1))
        n = nplus1
      goto 10
      end


      subroutine newene(e,path)
c*******************************************************************
c*    Initializes kcoefL() when the energy or the path length      *
c*    change                                                       *
c*    Input:                                                       *
c*      e -> kinetic energy in eV                                  *
c*      path -> path length in g/cm^2                              *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*******************************************************************
      implicit none
      real*8 e,path
      integer*4 maxnkl,nkl
      real*8 kl,eini,efin
      parameter (maxnkl=20000)
      common /lewdat/ kl(maxnkl),eini,efin,nkl
      real*8 energ,rang

      call setgls(1)
      nkl = 0
      eini = e
      efin = energ(rang(e)-path)
      end


      real*8 function kcoefL(n)
c*******************************************************************
c*    Gives the n-th coefficient of the Lewis series expansion,    *
c*    that is, K=Integ[ds/lambdan(s)] lambdan being the n-th       *
c*    transport MFP                                                *
c*                                                                 *
c*    Input:                                                       *
c*      n -> the order of the coefficient sought                   *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> newene() must be called first each time energy or path  *
c*         length change                                           *
c*      -> Note that, contrary to kcoef(), this routine does NOT   *
c*         return exp[-K], but K itself.                           *
c*      -> it stores previously calculated values in a common stack*
c*      -> 1st collision substraction is NOT performed             *
c*******************************************************************
      implicit none
      integer*4 n
      integer*4 maxnkl,nkl
      real*8 kl,eini,efin
      parameter (maxnkl=20000)
      common /lewdat/ kl(maxnkl),eini,efin,nkl
      integer*4 i,error
      real*8 integ,accu,zero,gls,infty
      parameter (accu=1.d-6,infty=1.0d30,zero=1.d-10)
      external gls

      if (n.eq.0) then
        kcoefL = 0.d0
        return
      endif
      if (n.gt.maxnkl) then
        write(*,*) 'kcoefL Error: stack overflow -> enlarge maxnkl'
        stop
      endif
      if (n.gt.nkl) then
com: loading the stack with new terms
        do 10 i=nkl+1,n
          call setgls(0)
          if (efin.gt.zero) then
            call gabq(gls,efin,eini,integ,accu,error)
          else
            error = 0
            integ = infty
          endif
          if (error.ne.0) then
            write(*,*)
     &      'kcoefL Warning: integration reported low accuracy;'
            write(*,*)
     &      '                i,efin,eini,integ: '
            write(*,'(1x,i5,1x,3(1pe10.3,1x))') i,efin,eini,integ
          endif
          kl(i) = integ
 10     continue
        nkl = n
      endif
      kcoefL = kl(n)
      end


      real*8 function gls(e)
c*******************************************************************
c*    Function to be integrated by kcoefL().                       *
c*                                                                 *
c*    Input:                                                       *
c*      e -> kinetic energy in eV                                  *
c*    Output:                                                      *
c*      1/(lambdan*StopPow) in eV^-1                               *
c*    Comments:                                                    *
c*      -> initot() must be called before 1st call                 *
c*      -> setgls() must be called previously to set the coef order*
c*      -> calling gls() after the 1st call to setgls(reset=0)     *
c*         produces the trasport coef of 1st order --see setgls()  *
c*      -> gls() uses the same approach as wtc(); however, since   *
c*         Lewis scheme requires an integral over energy, gls does *
c*         not call the latter to get rid of the repetitive calls  *
c*         to iniwtc() that the use of wtc() would require         *
c*******************************************************************
      implicit none
      real*8 e
      integer*4 n
      real*8 nn1,mess,polgam
      common /glsd/ nn1,mess,polgam,n
      real*8 aw,dbesk1,z,ilamtt,awtot,cstpow,rstpow

      aw = awtot(e)
      z = 2.d0*dsqrt(nn1*aw)
      gls = (1.d0-z*dbesk1(z)*(1.d0+aw*mess))*ilamtt(e)/
     &      (cstpow(e)+rstpow(e))
      end


      subroutine setgls(reset)
c*******************************************************************
c*    Sets some quantities used to calculate transport coefs       *
c*    as energy dependent functions.                               *
c*    This routine must be called when the _order_ of the term     *
c*    to calculate -with gls(energy)- changes, but not when the    *
c*    energy changes                                               *
c*                                                                 *
c*    Input:                                                       *
c*      reset -> 1: reset term order; 0: prepare for next order.   *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> each call increases by one the order of the term being  *
c*         prepared for gls() while reset=0. If reset=1 the system *
c*         is reset so that next call with reset=0 will prepare gls*
c*         to deliver term #1                                      *
c*******************************************************************
      implicit none
      integer reset
      integer*4 n
      real*8 nn1,mess,polgam
      common /glsd/ nn1,mess,polgam,n
      real*8 eulerc
      parameter (eulerc=0.57721566490153286d0)

      if (reset.eq.1) then
        n = 0
        polgam = -eulerc
        nn1 = 0.d0
        mess = 0.d0
        return
      endif
      n = n+1
      polgam = polgam+1.d0/n
      nn1 = n*dble(n+1)
      mess = 2.d0*nn1*(polgam-0.5d0*dlog(nn1))
      end


      real*8 function ssolv(deltak,energy)
c*******************************************************************
c*    Gives the path length that produces a given amount of        *
c*    scattering strength (dK=integ[ds/lambda1(s)])within the CSDA *
c*    to account for the variation of path length with energy.     *
c*                                                                 *
c*    Input:                                                       *
c*      deltak -> value of K to be matched.                        *
c*      energy -> initial kinetic energy of the particle (eV).     *
c*    Output:                                                      *
c*      -> path length in g/cm^2                                   *
c*    Comments:                                                    *
c*      -> inimat() must be called first                           *
c*      -> if a solution does not exist, returns -1.0d0            *
c*******************************************************************
      implicit none
      real*8 deltak,energy

      real*8 emin,emax,einf,esup,epinf,epsup,eminph
      common /elimit/ emin,emax,eminph,einf,esup,epinf,epsup

      integer worse
      real*8 ideltk,smax,rang,kcoefL,s,energ,f,precis,best,scptot
      real*8 accu,zero,infty,shigh,slow
      parameter (accu=1.0d-5,infty=1.0d30,zero=1.0d-30)

      if (deltak.lt.zero) then
        ssolv = 0.0d0
        return
      endif
      ideltk = 1.0d0/deltak

c     *** Make sure there is a solution:
      smax = rang(energy)
      call newene(energy,smax)
      if (kcoefL(1).lt.deltak) then
c       *** K is too large => no solution, so return negative value
c         * to indicate isotropic scattering:
        ssolv = -1.0d0
        return
      endif

c     *** Otherwise use Newton's method to find the root of
c       * f(s) = integ[ds' lambda1(s')]^s - K
c       * with df/ds = lambda1(s').
c       * Since f(0)<0 and f(s) increases and is, almost always, convex
c       * we take the initial guess s=smax:
      s = smax
      worse = 0
      best = infty
 10   continue
        call newene(energy,s)
        f = kcoefL(1)-deltak
        precis = dabs(f*ideltk)
        if (precis.lt.accu) then
          ssolv = s
          return
        endif
        if (precis.lt.best) then
          best = precis
        else
          worse = worse+1
          if (worse.ge.3) goto 20
        endif
        s = s-f/scptot(energ(smax-s))
c       *** Confine s:
        if (s.gt.smax) goto 20
        if (s.lt.0.0d0) s = 0.0d0
      goto 10

 20   continue
c     *** Newton didn't work; go for brute force (bipartition):
      slow = 0.0d0
      shigh = smax

 30   continue
        s = 0.5d0*(shigh+slow)
        call newene(energy,s)
        f = kcoefL(1)-deltak
        precis = dabs(f*ideltk)
        if (precis.lt.accu) then
          ssolv = s
          return
        endif
        if (f.gt.0.d0) then
          shigh = s
        else
          slow = s
        endif
      goto 30
      end


c* end of file *****************************************************

